package benefactorkindboxreqhandler

import (
	"net/http"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/kind_box_req"
	"git.gocasts.ir/ebhomengo/niki/pkg/claim"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

// delete godoc
// @Summary      delete kindboxreq by benefactor
// @Description  This endpoint is used to delete benefactor's kindboxreq at pending status
// @Tags         KindBoxReq
// @Accept       json
// @Produce      json
// @Param        id path int true "Kind box request ID"
// @Success      200  {object}  param.KindBoxReqDeleteResponse
// @Failure      400  {string}  "Bad request"
// @Security     AuthBearerBenefactor
// @Router       /benefactor/kindboxreqs/{id} [delete].
func (h Handler) Delete(c echo.Context) error {
	req := param.KindBoxReqDeleteRequest{}
	if bErr := echo.PathParamsBinder(c).Uint("id", &req.KindBoxReqID).BindError(); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}
	claims := claim.GetClaimsFromEchoContext(c)
	req.BenefactorID = claims.UserID
	if fieldErrors, err := h.benefactorKindBoxReqVld.ValidateDeleteRequest(req); err != nil {
		msg, code := httpmsg.Error(err)

		return c.JSON(code, echo.Map{
			"message": msg,
			"errors":  fieldErrors,
		})
	}
	resp, dErr := h.benefactorKindBoxReqSvc.Delete(c.Request().Context(), req)
	if dErr != nil {
		msg, code := httpmsg.Error(dErr)

		return echo.NewHTTPError(code, msg)
	}

	return c.JSON(http.StatusOK, resp)
}
