package mysqladdress

import (
	"context"
	"time"

	"git.gocasts.ir/ebhomengo/niki/entity"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	"git.gocasts.ir/ebhomengo/niki/repository/mysql"
)

func (d *DB) GetAllCities(ctx context.Context) ([]entity.City, error) {
	const op = "mysqladdress.GetAllCities"

	query := `SELECT * FROM cities;`
	//nolint
	stmt, err := d.conn.PrepareStatement(ctx, mysql.StatementKeyCityGetAll, query)
	if err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgCantPrepareStatement).WithKind(richerror.KindUnexpected)
	}

	rows, err := stmt.QueryContext(ctx)
	if err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
	}
	defer rows.Close()

	cities := make([]entity.City, 0)
	for rows.Next() {
		city, err := scanCity(rows)
		if err != nil {
			return nil, richerror.New(op).WithErr(err).
				WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
		}

		cities = append(cities, city)
	}

	if err = rows.Err(); err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
	}

	return cities, nil
}

func scanCity(scanner mysql.Scanner) (entity.City, error) {
	var createdAt, updatedAt time.Time
	var city entity.City

	err := scanner.Scan(&city.ID, &city.Name, &city.ProvinceID, &createdAt, &updatedAt)

	return city, err
}
