package mysqladmin

import (
	"context"
	"database/sql"
	"errors"
	"time"

	"git.gocasts.ir/ebhomengo/niki/entity"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	"git.gocasts.ir/ebhomengo/niki/repository/mysql"
)

func (d *DB) GetAddressByID(ctx context.Context, id uint) (*entity.Address, error) {
	const op = "mysqladdress.IsExistAddressByID"

	query := `select * from addresses where id = ? and deleted_at is null`
	//nolint
	stmt, err := d.conn.PrepareStatement(ctx, mysql.StatementKeyAddressGetByID, query)
	if err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgCantPrepareStatement).WithKind(richerror.KindUnexpected)
	}

	row := stmt.QueryRowContext(ctx, id)
	address, err := scanAddress(row)
	if err != nil {
		sErr := sql.ErrNoRows
		//TODO-errorsas: second argument to errors.As should not be *error
		//nolint
		if errors.As(err, &sErr) {
			return nil, nil
		}

		// TODO - log unexpected error for better observability
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgCantScanQueryResult).WithKind(richerror.KindUnexpected)
	}

	return &address, nil
}

func scanAddress(scanner mysql.Scanner) (entity.Address, error) {
	var createdAt, updatedAt time.Time
	var deletedAt sql.NullTime
	var address entity.Address

	err := scanner.Scan(&address.ID, &address.PostalCode, &address.Address, &address.Lat, &address.Lon,
		&address.Name, &address.CityID, &address.ProvinceID, &address.BenefactorID,
		&createdAt, &updatedAt, &deletedAt)

	return address, err
}
