package benefactorkindboxvalidator

import (
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/kind_box"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateRegisterEmptyingRequest(req param.KindBoxRegisterEmptyingRequest) (map[string]string, error) {
	const op = "benefactorkindboxvalidator.ValidateRegisterEmptyingRequest"

	if err := validation.ValidateStruct(&req,
		validation.Field(&req.BenefactorID, validation.Required,
			validation.By(v.doesBenefactorExist)),

		validation.Field(&req.KindBoxID, validation.Required,
			validation.By(v.doesBenefactorKindBoxExist(req.BenefactorID)),
			validation.By(v.doesKindBoxHaveDeliveredStatus)),

		validation.Field(&req.ReturnAddressID, validation.Required,
			validation.By(v.doesAddressExist(req.BenefactorID))),

		validation.Field(&req.ReturnReferDate, validation.Required,
			validation.By(v.isDateValid)),

		validation.Field(&req.ReturnReferTimeID, validation.Required,
			validation.By(v.isReferTimeIDValid)),
	); err != nil {
		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	return map[string]string{}, nil
}
